import React, { useState, useEffect, useRef } from 'react';
import { DragDropContext, Droppable, Draggable } from '@hello-pangea/dnd';
import Card from '@/Components/Card';
import { motion } from "framer-motion";
import Button from '@/Components/Buttons/Button';
import StageForm from './StageForm';
import toast from 'react-hot-toast';

export default function SortableColumns({ funnel, handleNewStage, exitActions, filters }) {
    const [stages, setStages] = useState(funnel?.stages || []);
    const [columns, setColumns] = useState(stages);
    const [loading, setLoading] = useState(false);

    const scrollContainerRef = useRef(null);

    const handleDragEnd = async (result) => {
        setLoading(true);
        if (!result.destination) return;

        const reorderedColumns = Array.from(columns);
        const [movedColumn] = reorderedColumns.splice(result.source.index, 1);
        reorderedColumns.splice(result.destination.index, 0, movedColumn);

        const updatedColumns = reorderedColumns.map((col, index) => ({
            ...col,
            position: index + 1
        }));

        setColumns(updatedColumns);

        try {
            const response = await fetch(route('stage.update.positions', funnel?.id), {
                method: 'POST',
                headers: {
                    'X-Requested-With': 'XMLHttpRequest',
                    'X-CSRF-TOKEN': document.querySelector('meta[name="csrf-token"]').getAttribute('content'),
                    'Content-Type': 'application/json',
                },
                body: JSON.stringify({
                    stages: updatedColumns.map(col => ({ id: col.id, position: col.position }))
                }),
            });

            if (!response.ok) {
                throw new Error('Erro ao atualizar as posições.');
            }
        } catch (error) {
            toast.error(error.message);
        } finally {
            setLoading(false);
        }
    };

    useEffect(() => {
        if (scrollContainerRef.current) {
            scrollContainerRef.current.scrollTo({
                left: scrollContainerRef.current.scrollWidth,
                behavior: 'smooth',
            });
        }
    }, [columns]);

    useEffect(() => {
        setStages(funnel?.stages || []);
        setColumns(funnel?.stages || []);
    }, [funnel]);

    return (
        <div ref={scrollContainerRef} className="flex gap-1 overflow-x-auto max-w-full">
            <DragDropContext onDragEnd={handleDragEnd}>
                <Droppable droppableId="columns" direction="horizontal" type="COLUMN">
                    {(provided) => (
                        <div className="flex gap-1" {...provided.droppableProps} ref={provided.innerRef}>
                            {columns.map((column, index) => (
                                <Draggable key={column.id} draggableId={String(column.id)} index={index}>
                                    {(provided) => (
                                        <div ref={provided.innerRef} {...provided.draggableProps} {...provided.dragHandleProps}>
                                            <motion.div key={index} initial={{ opacity: 0 }} animate={{ opacity: 1 }} transition={{ duration: 0.5, delay: index * 0.09 }}>
                                                <Card className="w-[350px] shrink-0 h-[86vh]">
                                                    <StageForm stage={column} stages={stages} exitActions={exitActions} filters={filters} />
                                                </Card>
                                            </motion.div>
                                        </div>
                                    )}
                                </Draggable>
                            ))}

                            {provided.placeholder}

                            {!loading && (
                                <motion.div initial={{ opacity: 0 }} animate={{ opacity: 1 }} transition={{ duration: 0.5, delay: 0.08 }}>
                                    <div className="w-[350px] shrink-0 h-[86vh] flex flex-col justify-center items-center">
                                        <p className="text-lg font-bold">Adicionar nova etapa</p>
                                        <p className="text-[.8rem] text-center">As etapas do funil representam os passos em seu processo de vendas</p>
                                        <Button className="mt-5" onClick={handleNewStage}>Criar etapa</Button>
                                    </div>
                                </motion.div>
                            )}
                        </div>
                    )}
                </Droppable>
            </DragDropContext>
        </div>
    );
};
